@@ -1,4 +1,4 @@
-void
+static void
 fullscreen(void) {
 	Client *c;
 	for(c = clients; c; c = c->next)
