@@ -75,4 +75,7 @@ void madtty_color_set(WINDOW *win, short fg, short bg);
 void madtty_scroll(madtty_t *, int rows);
 void madtty_noscroll(madtty_t *);
 
+void madtty_bell(madtty_t *, bool bell);
+void madtty_togglebell(madtty_t *);
+
 #endif /* MADTTY_MADTTY_H */
